//
// Simple GLUT Viewer [uses GLM Library]
//
// Reto Ltolf,1999,2001.
// E-mail: luetolf@inf.ethz.ch
// URL:    http://graphics.ethz.ch/
//


#include "viewer.h"


//globals
GLMmodel *model;
GLuint    material_mode = _MAT_NONE;
GLuint    model_list = 0;
GLfloat   scale_factor;
GLboolean use_facet_normals = GL_FALSE;
char   *model_file = NULL;


//init functions
void func_init(void)
{
  //read the model
  model = glmReadOBJ(model_file);
  scale_factor = glmUnitize(model);
  glmFacetNormals(model);
  glmVertexNormals(model, _SMOOTHING_ANGLE);
  
  //create new display lists
  if (model->nummaterials > 0) material_mode = _MAT_MATERIAL;
  func_create_list();

  //set GL options
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
  glEnable(GL_COLOR_MATERIAL);
  glColor3f(0.5, 0.5, 0.5);
  glClearColor(0.0, 0.0, 0.0, 1.0);

  //init trackball
  TB_init(GLUT_MIDDLE_BUTTON);
  TB_animate(GL_TRUE);
}

void func_create_list(void)
{
  GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
  GLfloat diffuse[] = { 0.8, 0.8, 0.8, 1.0 };
  GLfloat specular[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat shininess = 65.0;

  if (model_list!=0) glDeleteLists(model_list,1);

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
  glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

  //generate list
  if (material_mode == _MAT_NONE) {
    if (use_facet_normals) model_list = glmList(model, GLM_FLAT);
    else model_list = glmList(model, GLM_SMOOTH);
  }
  else if (material_mode == _MAT_COLOR) {
    if (use_facet_normals) model_list = glmList(model, GLM_FLAT|GLM_COLOR);
    else model_list = glmList(model, GLM_SMOOTH|GLM_COLOR);
  }
  else if (material_mode == _MAT_MATERIAL) {
    if (use_facet_normals) model_list = glmList(model, GLM_FLAT|GLM_MATERIAL);
    else model_list = glmList(model, GLM_SMOOTH|GLM_MATERIAL);
  }
}

void func_create_menu(void)
{
  int main_menu;

  main_menu = glutCreateMenu(callback_menu_main);
  glutAddMenuEntry("Viewer with Trackball", _MENU_EMPTY);
  glutAddMenuEntry("", _MENU_EMPTY);
  glutAddMenuEntry("[m] Toggle  none / color / material", _MENU_MAT);
  glutAddMenuEntry("[n] Toggle  facet / smooth normals", _MENU_NORM);
  glutAddMenuEntry("[w] Toggle  wireframe / filled", _MENU_WIRE);
  glutAddMenuEntry("", _MENU_EMPTY);
  glutAddMenuEntry("[s] Scale model smaller", _MENU_SCALE_DOWN);
  glutAddMenuEntry("[S] Scale model larger", _MENU_SCALE_UP);
  glutAddMenuEntry("", _MENU_EMPTY);
  glutAddMenuEntry("[h] Print help", _MENU_HELP);
  glutAddMenuEntry("", _MENU_EMPTY);
  glutAddMenuEntry("[q] Quit", _MENU_QUIT);
  glutSetMenu(main_menu);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

void func_action_help(void)
{
  printf("\nhelp\n\n");
  printf(" h - this help text...\n");
  printf(" m - toggle rendering with no/color/material properties\n");
  printf(" n - toggle using facet/smooth normals\n");
  printf(" w - toggle wireframe/filled rendering\n");
  printf(" s - scale model smaller\n");
  printf(" S - scale model larger\n");
  printf(" q - quit program\n\n");
}

void func_action_material(void)
{
  material_mode++;
  if (material_mode > _MAT_MATERIAL) material_mode = _MAT_NONE;
  func_create_list();
}

void func_action_normals(void)
{
  use_facet_normals = !use_facet_normals;
  func_create_list();
}

void func_action_wire(void)
{
  GLint params[2];
  glGetIntegerv(GL_POLYGON_MODE, params);
  if (params[0] == GL_FILL) glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  else glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
}

void func_action_scaleDown(void)
{
  glmScale(model, 0.8);
  func_create_list();
}

void func_action_scaleUp(void)
{
  glmScale(model, 1.2);
  func_create_list();
}

void func_action_quit(void)
{
  glmDelete(model);
  exit(0);
}


//callback functions
void callback_display(void)
{
  //clear frame and depth buffer
  /// Add your code here
  
  
  //prepare matrix stack and load rotation matrix 
  //from trackball module to matrix stack
  /// Add your code here


  //call display list generated in 'func_create_list()'
  /// Add your code here
  
  
  //restore matrix stack
  /// Add your code here


  //swap frame buffers
  /// Add your code here
  
}

void callback_reshape(int w, int h)
{
  //give reshape information to trackball module
  TB_reshape(w, h);

  //adapt projection
  glViewport(0, 0, w, h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0, (GLfloat)h/(GLfloat)w, 1.0, 128.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0.0, 0.0, -3.0);
}

void callback_keyboard(unsigned char key, int x, int y)
{
  switch(key) {
  case 'h':
    func_action_help();
    break;
  case 'm':
    func_action_material();
    break;
  case 'n':
    func_action_normals();
    break;
  case 'w':
    func_action_wire();
    break;
  case 's':
    func_action_scaleDown();
    break;
  case 'S':
    func_action_scaleUp();
    break;
  case 'q':
    func_action_quit();
  default:
    break;
  };
  glutPostRedisplay();
}

void callback_mouse(int button, int state, int x, int y)
{
  //give control to trackball module
  TB_mouse(button, state, x, y);
}

void callback_motion(int x, int y)
{
  //give control to trackball module
  TB_motion(x, y);
}

void callback_menu_main(int val)
{
  switch(val) {
  case _MENU_MAT:
    func_action_material();
    break;
  case _MENU_NORM:
    func_action_normals();
    break;
  case _MENU_WIRE:
    func_action_wire();
    break;
  case _MENU_SCALE_DOWN:
    func_action_scaleDown();
    break;
  case _MENU_SCALE_UP:
    func_action_scaleUp();
    break;
  case _MENU_HELP:
    func_action_help();
    break;
  case _MENU_QUIT:
    func_action_quit();
  default:
    break;
  };
  glutPostRedisplay();
}


//main
int main(int argc, char** argv)
{
  glutInit(&argc, argv);

  //test & get argument:
  if (argc<2) {
    model_file = "data/castle.obj";
    printf("no model-file.obj specified as argument.\n");
    printf("using %s.\n", model_file);
  }
  else {
	model_file = argv[1];
  }

  //init glut window:
  glutInitWindowSize(_X_SIZE, _Y_SIZE);
  glutInitDisplayMode(GLUT_RGB|GLUT_DEPTH|GLUT_DOUBLE);
  glutCreateWindow("Viewer with Trackball");

  //init all the rest:
  func_init();
  func_create_menu();

  //register callback functions:
  glutDisplayFunc(callback_display);
  glutReshapeFunc(callback_reshape);
  glutKeyboardFunc(callback_keyboard);
  glutMouseFunc(callback_mouse);
  glutMotionFunc(callback_motion);
  glutPassiveMotionFunc(NULL);
  glutVisibilityFunc(NULL);
  glutEntryFunc(NULL);
  glutSpecialFunc(NULL);
  glutIdleFunc(NULL);

  //enter glut main loop:
  glFlush();
  glutMainLoop();
  return(0);
}
